package com.jz.dm.controller;

import com.jz.common.utils.Result;
import com.jz.dm.models.req.make.*;
import com.jz.dm.service.ProducerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.controller
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ProducerController
 * @DATE: 2020-12-23/16:44
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@RestController
@RequestMapping("api/producer")
@Api(tags = "Api制作Controller")
public class ProducerController {

    @Autowired
    private ProducerService producerService;


    /**
     * @Description:获取文件夹列表
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取文件夹列表")
    @GetMapping(value = "/getFileCatalog")
    public Mono<Result> getFileCatalog(@RequestParam(name = "projectId")Long projectId) {
        return Mono.fromSupplier(() -> producerService.getFileCatalog(projectId));
    }
    /**
     * @Description:创建文件夹
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("创建项目文件夹")
    @PostMapping(value = "/createProjectFolder")
    public Mono<Result> createProjectFolder(@RequestParam(name = "projectId")Long projectId,
                                            @RequestParam(name = "parentId")Long parentId,
                                            @RequestParam(name = "currentUser") String currentUser ) {
        return Mono.fromSupplier(() -> producerService.createProjectFolder(projectId,parentId,currentUser));
    }
    /**
     * @Description:Api制作(自定义DMP)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(自定义DMP)")
    @PostMapping(value = "/addCustomApi")
    public Mono<Result> makeCustomApi(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addCustomApi(req)));
    }

    /**
     * @Description:Api制作(数据银行)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(数据银行)")
    @PostMapping(value = "/addDataBankApiInfo")
    public Mono<Result> addDataBankApiInfo(@RequestBody @Valid MakeDataBankApiReq req) {
        return Mono.fromSupplier(() -> producerService.addDataBankApiInfo(req));
    }

    /**
     * @Description:Api制作(实时接入)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(实时接入)")
    @PostMapping(value = "/realMakeApi")
    public Mono<Result> realCustomApi(@RequestBody @Valid MakeRealCustomApiReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addRealCustomApi(req)));
    }
    /**
     * @Description:Api制作(大数据查询/标签查询)
     * @return:
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(大数据查询/标签查询)")
    @PostMapping(value = "/makeBigDataApi")
    public Mono<Result> bigDataMakeApi(@RequestBody @Valid MakeBigDataApiReq req) {
        return Mono.fromSupplier(() -> producerService.addBigDataMakeApi(req));
    }

    /**
     * @Description:保存API基本信息/自定义API
     * @return: (保存API基本信息)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("保存API基本信息")
    @PostMapping(value = "/saveInterface")
    public Mono<Result> saveInsetInterface(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> producerService.saveInterfaceAPi(req));
    }
    /**
     * @Description:Api制作(大数据查询/标签查询)保存修改的api信息
     * @return: (自定义更新同时使用)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("保存修改的api信息")
    @PostMapping(value = "/saveUpdate")
    public Mono<Result> saveUpdateAPiInfo(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> producerService.saveUpdateAPiInfo(req));
    }


    /**
     * @Description:获取数据源配置
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取数据源配置")
    @PostMapping(value = "/getSourceConfig")
    public Mono<Result> getSourceConfig(@RequestBody @Valid SourceConfigReq req) {
        return Mono.fromSupplier(() -> producerService.getSourceConfig(req));
    }
    /**
     * @Description:获取索引
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取索引")
    @GetMapping(value = "/getIndicess")
    public Mono<Result> getSourceWarehouseList(@RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.getSourceWarehouseList(id));
    }
    /**
     * @Description:获取某个索引下的type
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取某个索引下的type")
    @GetMapping(value = "/getIndicesTypes")
    public Mono<Result> getIndicesTypes(@RequestParam(name = "indices") String indices,
                                        @RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.getIndicesTypes(indices,id));
    }
    /**
     * @Description:获取某个索引 type的fields
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取某个索引 type的fields")
    @GetMapping(value = "/getIndicesTypesFields")
    public Mono<Result> getIndicesTypesFields(@RequestParam(name = "indices") String indices,
                                              @RequestParam(name = "type") String type,
                                              @RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.getIndicesTypesFields(indices,type,id));
    }


    @ApiOperation("保存数据到redis中")
    @GetMapping(value = "/saveRedis")
    public Result saveObjOnRedis() {
        return producerService.saveObjOnRedis();
    }

    @ApiOperation("获取redis中的数据")
    @GetMapping(value = "/getRedisObj")
    public Result getObjOnRedis() {
        return  producerService.getObjOnRedis();
    }
}
