package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.utils.Result;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.config.ReentrantRedisLock;
import com.jz.dm.mapper.ApiReqLogMapper;
import com.jz.dm.models.domian.ApiReqLog;
import com.jz.dm.models.req.LogInfoDetailReq;
import com.jz.dm.models.req.LogInfoListReq;
import com.jz.dm.service.ApiLogService;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiLogServiceImpl
 * @DATE: 2020-12-25/14:29
 * @DAY_NAME_SHORT: 周五
 * @Description:
 **/
@Service("apiLogService")
@Slf4j
public class ApiLogServiceImpl implements ApiLogService {


    @Resource
    private ApiReqLogMapper apiReqLogMapper;
    // 加锁
    @Autowired
    private ReentrantRedisLock lock;
    /**
     * 日志信息列表
     * @param req
     * @return
     */
    @Override
    public IPage<ApiReqLog> listApiLog(LogInfoListReq req) {
        IPage<ApiReqLog> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiReqLog> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getApiKey())){
            query.eq("api_key",req.getApiKey());
        }
        if (StringUtils.isNotBlank(req.getStatus())){
           query.eq("status",req.getStatus());
        }
        query.eq("is_deleted",0);
        query.orderByDesc("create_date");
        return apiReqLogMapper.selectPage(page, query);
    }

    /**
     * 查询日志详情
     * @param req
     * @return
     */
    @Override
    public Result getApiLogDetail(LogInfoDetailReq req) {
        return Result.of_success(apiReqLogMapper.selectById(req.getId()));
    }

    /**
     * 保存日志
     * @param reqLog
     */
    @Override
    public void insetLogInfo(ApiReqLog reqLog) {
        apiReqLogMapper.insert(reqLog);
    }
    /**
     * api计量统计
     * @param date
     * @return
     */
    @Override
    public Result countAPiCallStat(String date ) {
        Date dateParam =null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("YYYY-MM");
            dateParam = dateFormat.parse(date);
        }catch (Exception ex){
           log.error("时间转换异常:{}",ex.getMessage());
        }
      return Result.of_success(apiReqLogMapper.countApiCallMonthData(dateParam));
    }

    /**
     * 根据id更新日志
     * @param id
     * @param jsonObject
     */
    @Override
    public void updateLog(Long id, JSONObject jsonObject) {
        try {
            /*ApiReqLog apiReqLog = apiReqLogMapper.maxId(id);*/
            ApiReqLog apiReqLog = apiReqLogMapper.selectById(id);
            if (null != apiReqLog){
                UpdateWrapper<ApiReqLog> update = new UpdateWrapper<>();
                if (200 == jsonObject.getInt("code")){
                    update.set("status", GeneralStatusTypeEnum.SUCCEED);
                }else {
                    update.set("status", GeneralStatusTypeEnum.FAIL);
                }
                update.set("response_params",jsonObject.toString());
                update.set("update_date",new Date());
                update.set("request_end_time",new Date());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(apiReqLog.getRequestStartTime());
                Long time = System.currentTimeMillis() - calendar.getTimeInMillis() / 1000;
                double timeConsuming = time.doubleValue();
                update.set("request_consuming",timeConsuming);
                update.eq("id",id);
                if (apiReqLogMapper.update(null,update) == 0){
                    log.info("~~~~~~~~~~~更新日志信息失败~~~~~~~~");
                }
            }else {
                log.info("id为:{}",id+"--------------日志信息不存在");
            }
        }catch (Exception ex){
            log.error("更新日志返回信息异常:{}",ex.getMessage());
        }
    }


}
