package com.jz.dm.service.request;

import com.alibaba.fastjson.JSONObject;
import com.jz.common.utils.RedisUtils;
import com.jz.dm.common.enums.GatewayResultCode;
import com.jz.dm.common.enums.apiInterface.ApiInfoOutTypeEnum;
import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.OpenApiRequest;
import com.jz.dm.common.util.OpenApiResponse;
import com.jz.dm.gateway.OpenApiService;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.service.ApiInterfaceService;
import com.jz.dm.web.annotation.AccessLimit;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Map;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.request
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiQueryTestService
 * @DATE: 2021-1-25/10:13
 * @DAY_NAME_SHORT: 周一
 * @Description:
 **/
@Service("apiQueryTestService")
@Slf4j
public class ApiQueryTestService extends ApiParamVerify implements OpenApiService {
    @Override
    public String getOpenApiMethod() {
        return "test";
    }

    @Override
    public String getOpenApiVersion() {
        return "v1.0.0";
    }

    @Autowired
    private  ApiQueryService apiQueryService;
    @Resource
    private ApiInterfaceService apiInterfaceService;
    @Autowired
    private RedisUtils redisUtils;
    /**
     * API测试实现
     * @param request
     * @param response
     */
    @Override
    @Transactional(rollbackFor = Exception.class,propagation = Propagation.REQUIRED)
    @AccessLimit(sec = 1,limit = 1000)
    public void doService(OpenApiRequest request, OpenApiResponse response) {
        JSONObject parameter = JSONObject.parseObject(request.getOpenApiParams());
        ApiInterface apiInterface = apiInterfaceService.getApiInfo(request.getApiKey());
        verifyApiInterface(apiInterface);
        String reqParams = parameter.getString("reqParams");
        JSONObject jsonParams = null;
        Map paramMap = null;
        if (StringUtils.isNotBlank(reqParams)) {
            jsonParams = JSONObject.parseObject(reqParams);
            paramMap = JSONObject.parseObject(jsonParams.getString("request_fileds"), Map.class);
            jsonParams.put("is_test",true);
        }
        //取出缓存数据
        // String redisReqParam = redisUtils.get(request.getApiKey());
         String redisReqParam = null;
        if (StringUtils.isNotBlank(redisReqParam)) {//redis中存在
            //解析出API制作成功时的参数配置
            JSONObject jsonObject = JSONObject.parseObject(redisReqParam);
            String targetUrl = jsonObject.getString("targetUrl");
            String outputType = jsonObject.getString("outputType");
            if (StringUtils.isBlank(outputType)) {
                outputType = ApiInfoOutTypeEnum.JSON.name();
            }
            if (StringUtils.isBlank(targetUrl)) {
                throw new GatewayException(GatewayResultCode.REQUEST_PARAM_EMPTY);
            }
            apiQueryService.dataTableSelect(outputType, targetUrl,  paramMap,
                    jsonParams, apiInterface, response);
        } else {//不存在查询数据库
            apiQueryService.dataTableSelect(apiInterface.getOutputType(),
                    apiInterface.getTargetUrl(),  paramMap, jsonParams, apiInterface, response);
        }
    }
}
