package com.jz.dm.mall.moduls.controller.company;

import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.company.bean.CompanyAddReq;
import com.jz.dm.mall.moduls.controller.company.bean.CompanyUpdateReq;
import com.jz.dm.mall.moduls.service.CompanyAuthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.servlet.http.HttpServletRequest;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.mall.moduls.controller.company
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: CompanyAuthController
 * @USER: key
 * @DATE: 2020-12-2/16:33
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@RestController
@RequestMapping("/companyAuth")
@Api(tags = "企业认证controller")
public class CompanyAuthController {

    @Autowired
    private CompanyAuthService companyAuthService;


    /**
    * @Description: 添加企业认证信息
    * @Author: Mr.zhang
    * @Date: 2020-12-2
    */
    @PostMapping("/add")
    @ApiOperation(value = "企业信息提交")
    public Mono<Result> addCompanyAuth(@RequestBody @Validated CompanyAddReq req,HttpServletRequest request) {
        return Mono.fromSupplier(() -> companyAuthService.addCompanyData(req,request));
    }

    /**
     * @Description: 企业认证信息查询
     * @Author: Mr.zhang
     * @Date: 2020-12-2
     */
    @GetMapping("/findCompany")
    @ApiOperation(value = "企业认证信息查询")
    public Result selectCompany(@RequestParam(value = "type") String type,HttpServletRequest request) {
        return companyAuthService.selectCompany(type,request);
    }
    /**
     * @Description: 企业认证信息更新
     * @Author: Mr.zhang
     * @Date: 2020-12-2
     */
    @GetMapping("/updateCompany")
    @ApiOperation(value = "企业认证信息更新")
    public Mono<Result> updateCompany(@RequestBody @Validated CompanyUpdateReq req,HttpServletRequest request) {
        return Mono.fromSupplier(() ->companyAuthService.updateCompanyInfo(req,request));
    }
    /**
     * @Description: 企业认证信息更新
     * @Author: Mr.zhang
     * @Date: 2020-12-2
     */
    @GetMapping("/checkCompany")
    @ApiOperation(value = "企业认证信息校验")
    public Mono<Result> checkCompany(HttpServletRequest request) {
        return Mono.fromSupplier(() ->companyAuthService.checkCompanyInfo(request));
    }

    @GetMapping("/getProvince")
    @ApiOperation(value = "获取所有省份")
    public Result getProvince() {
        Result result = companyAuthService.getProvince();
        return result;
    }

    @PostMapping("/getCity")
    @ApiOperation(value = "获取当前省份下的城市")
    public Result getCity(@RequestParam(name = "provinceCode") String provinceCode,
                          @RequestParam(name = "cityCode",required = false) String cityCode) {
        return companyAuthService.getCity(provinceCode,cityCode);
    }
}
