package com.jz.dm.mall.moduls.controller.customer;

import com.baomidou.mybatisplus.extension.api.R;
import com.jz.common.base.BaseController;
import com.jz.common.base.CurrentUser;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.SessionUtils;
import com.jz.dm.mall.moduls.controller.customer.bean.CustomerDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.CustomerRequest;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.common.utils.Result;
import com.jz.common.utils.StatusCode;
import com.jz.dm.mall.moduls.service.MallCustomerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 商城用户(MallCustomer)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@RestController
@RequestMapping("/mallCustomer")
@Api(tags = "商城用户--controller")
public class MallCustomerController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private MallCustomerService mallCustomerService;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 添加用户
     * @param customer
     * @return
     */
    @PostMapping("/saveCustomer")
    @ApiOperation(value = "注册用户", notes = "添加用户")
    public Result saveCustomer(@RequestBody CustomerRequest customer, HttpServletRequest request) throws Exception{
            Result result = mallCustomerService.saveCustomer(customer);
            return result;
    }


    /**
     * 获取用户信息
     * @return
     */
    @GetMapping("/getLoginUserName")
    @ApiOperation(value = "获取当前登录用户信息")
    public Result getLoginUserName (HttpServletRequest request) throws Exception{
        // 从redis中获取id
        try {
            MallCustomerApiDto mallCustomer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
//            Long customerId = 1L;
            Long customerId = mallCustomer.getCustomerId();
            MallCustomer user = mallCustomerService.selectByUser(customerId);
            return new Result(true, "获取用户信息成功！", StatusCode.OK, user);
        }catch (Exception e) {
            e.printStackTrace();
            return new Result(false, "获取用户信息失败", StatusCode.ERROR);
        }
    }

    @PostMapping(value = "/updatePassword")
    @ApiOperation(value = "密码修改")
    public Result updatePassword(@RequestParam(value = "oldPassWard") String oldPassword,
                                 @RequestParam(value = "newPassword") String newPassword,HttpServletRequest request) throws Exception {
        // 获取用户信息
        MallCustomer mallCustomer = (MallCustomer) getLoginUserName(request).getData();
        // 如果密码一致
        if (mallCustomer.getPassword().equals(oldPassword)) {
            // 修改密码
            Long customerId = mallCustomer.getCustomerId();
            mallCustomerService.updatePassword(customerId, newPassword);
            return new Result(true, "密码修改成功", StatusCode.OK);
        }
        return new Result(false, "密码修改失败", StatusCode.ERROR);
    }

    @PostMapping("/getMenu")
    @ApiOperation(value = "商城--角色展示菜单", notes = "用户角色展示菜单")
    public Result getMenu(@RequestParam(name = "userId") Long userId,
                          @RequestParam(name = "roleId") Long roleId) {
        Result menu = mallCustomerService.getMenu(userId, roleId);
        if (!StringUtils.isEmpty(menu)) {
            menu.setMsg("查询菜单成功!");
            menu.setCode(StatusCode.OK);
            return menu;
        }
        return new Result(false, "查询菜单失败!", StatusCode.ERROR);
    }
}