package com.jz.dm.mall.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.*;
import com.jz.common.constant.Constants;

import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.HttpsUtilsTest;
import com.jz.common.utils.Md5;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataGoodsDto;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataGoodsListDto;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DataApiUpReq;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DataGoodsListRequest;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DataGoodsRequest;
import com.jz.dm.mall.moduls.controller.goods.bean.request.MakeDataBankApiReq;
import com.jz.dm.mall.moduls.entity.DataGoods;
import com.jz.dm.mall.moduls.entity.DataGoodsApi;
import com.jz.dm.mall.moduls.entity.DataGoodsApiParams;
import com.jz.dm.mall.moduls.mapper.DataGoodsApiDao;
import com.jz.dm.mall.moduls.mapper.DataGoodsApiParamsDao;
import com.jz.dm.mall.moduls.mapper.DataGoodsDao;
import com.jz.dm.mall.moduls.service.DataGoodsService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据商品(DataGoods)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:30
 */
@Service("dataGoodsService")
public class DataGoodsServiceImpl implements DataGoodsService {
	private static final Logger logger = LoggerFactory.getLogger(DataGoodsServiceImpl.class);

	@Autowired
	private DataGoodsDao dataGoodsDao;

	@Autowired
	private DataGoodsApiParamsDao apiParamsDao;

	@Autowired
	private RedisTemplate redisTemplate;

	@Autowired
	private DataGoodsApiDao tDataGoodsApiDao;


	@Value("${domain.apigateway}")
	private String apiGateway;

	@Value("${token.dataBank}")
	private String dataBank;


    @Override
	public PageInfoResponse<DataGoodsListDto> findList(DataGoodsListRequest dataGoodsListRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DataGoodsListDto> pageInfoResponse = new PageInfoResponse<>();
		Map<String, Object> param = new HashMap<>();
		//商品分类id（行业）
		if (dataGoodsListRequest.getCategoryId() != null) {
			param.put("categoryId", dataGoodsListRequest.getCategoryId());
		}
		//数据商品名称
		if (StringUtils.isNotEmpty(dataGoodsListRequest.getDataName())) {
			param.put("dataName", dataGoodsListRequest.getDataName());
		}
		//数据类型：01api，02数据包
		if (StringUtils.isNotEmpty(dataGoodsListRequest.getDataType())) {
			param.put("dataType", dataGoodsListRequest.getDataType());
		}

		//价格类型：01免费，02收费
		if (StringUtils.isNotEmpty(dataGoodsListRequest.getPriceType())) {
			param.put("priceType", dataGoodsListRequest.getPriceType());
		}

		//更新时间排序
		if (dataGoodsListRequest.getUptTime() != null) {
			param.put("uptTime", dataGoodsListRequest.getUptTime());
		}
		// 使用人数
		if (dataGoodsListRequest.getUsePerson() != null) {
			param.put("usePerson", dataGoodsListRequest.getUsePerson());
		}
		// 综合排序
		if (StringUtils.isNotEmpty(dataGoodsListRequest.getSynthesizeRank())) {
			param.put("synthesizeRank", dataGoodsListRequest.getSynthesizeRank());
		}

		PageHelper.startPage(dataGoodsListRequest.getPageNum(), dataGoodsListRequest.getPageSize());
		List<DataGoodsListDto> list = dataGoodsDao.findList(param);
		for (DataGoodsListDto dataGoodsListDto : list) {
			dataGoodsListDto.getDataPrice().setScale(3);
		}
		PageInfo<DataGoodsListDto> pageInfo = new PageInfo<>(list);
		
		pageInfoResponse.setCode(Constants.SUCCESS_CODE);
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}
	

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DataGoodsService#add(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DataGoods> add(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();
		
		/*$*seDataGoodsIdCodeReplace*$*/
		//创建人
		//dataGoods.setCreateUser(getHttpRequestUserId(httpRequest));
		//创建时间
		dataGoods.setCreTime(new Date());
		
		dataGoodsDao.insertSelective(dataGoods);
		
		baseBeanResponse.setCode(Constants.SUCCESS_CODE);
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dataGoods);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DataGoodsService#edit(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DataGoods> edit(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();
		
		//修改人
		//dataGoods.setUpdateUser(getHttpRequestUserId(httpRequest));
		//修改时间
		dataGoods.setUptTime(new Date());
		
		dataGoodsDao.updateByPrimaryKeySelective(dataGoods);
		
		baseBeanResponse.setCode(Constants.SUCCESS_CODE);
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dataGoods);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DataGoodsService#findById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DataGoodsDto> findById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DataGoodsDto> baseBeanResponse = new BaseBeanResponse<>();
		
		DataGoodsDto DataGoodsDto = dataGoodsDao.findById(dataGoodsRequest.getDataGoodsId());
		
		baseBeanResponse.setCode(Constants.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(DataGoodsDto);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DataGoodsService#deleteById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dataGoodsDao.deleteByPrimaryKey(dataGoodsRequest.getDataGoodsId());
		
		baseResponse.setCode(Constants.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");
		
		return baseResponse;
	}

	/**
	 * 自营数据-上传api
	 *
	 * @param dataApiUpReq
	 */
	@Override
	@Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
	public Result uploadApi(DataApiUpReq dataApiUpReq) throws Exception  {
		// 获取当前用户
		MallCustomerApiDto customer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);

		// 校验参数
		Result checkParams = checkOut(dataApiUpReq);
		if (!checkParams.isSuccess()) {
			return Result.of_error("参数校验错误！");
		}
		// 调用gateway获取apiKey
		Result getApiKey = getApiKey(dataApiUpReq, customer.getCustomerId());
		String apiKey = (String) getApiKey.getData();
		if (apiKey == null) {
			return Result.of_error("获取apiKey失败！");
		}
		// 封装参数
		Result params = packParams(dataApiUpReq, customer.getCustomerId(), apiKey);
		if (!params.isSuccess()) {
			return Result.of_error("封装参数失败！");
		}
		return Result.of_success(ResultMsg.SUCCESS);
	}

	/**
	 * 校验参数
	 */
	public Result checkOut(DataApiUpReq dataApiUpReq) {
		if (dataApiUpReq == null) {
			return Result.error("上传失败！");
		}
		if (StringUtils.isEmpty(dataApiUpReq.getReturnType())) {
			return Result.error("输出类型不能为空");
		}
		if (StringUtils.isEmpty(dataApiUpReq.getJoinType())) {
			return Result.error("接入类型不存在！");
		}
		if (StringUtils.isEmpty(dataApiUpReq.getApiProtocl())) {
			return Result.error("传输方式不能为空！");
		}
		if (StringUtils.isEmpty(dataApiUpReq.getEncryMode())) {
			return Result.error("加密方式不能为空！");
		}
		return Result.ok();
	}

	/**
	 * 调用gateway获取apikey
	 */
	public Result getApiKey(DataApiUpReq dataApiUpReq, Long customerId) {
		// 调用gateway获取apiKey
		MakeDataBankApiReq apiReq = new MakeDataBankApiReq();

		// 必填参数
		apiReq.setFileId(1L);
		apiReq.setJoinType(dataApiUpReq.getJoinType());
		apiReq.setOutputType(dataApiUpReq.getReturnType());
		apiReq.setPage(dataApiUpReq.getPage()); // TODO
		apiReq.setReqType("DAY"); // TODO
		apiReq.setSign(getSign(dataApiUpReq.getJoinType())); // T
		apiReq.setTargetUrl(dataApiUpReq.getTargetUrl());
		apiReq.setTransMode(dataApiUpReq.getApiProtocl());
		apiReq.setSignType(dataApiUpReq.getEncryMode());
		apiReq.setOutputType(dataApiUpReq.getReturnType());
		// 非必填
		apiReq.setApiFunction(dataApiUpReq.getDataDesc());
		apiReq.setApiPath(dataApiUpReq.getApiUrl());
		apiReq.setCreateUser(customerId + "");
		apiReq.setReqFrequency(dataApiUpReq.getReqFrequency());
		apiReq.setTimeout(""); //TODO
		for (DataGoodsApiParams params : dataApiUpReq.getRequestParam()) {
			apiReq.setRequestParam(JSON.toJSONString(params));
		}
		for (DataGoodsApiParams params : dataApiUpReq.getResponseParam()) {
			apiReq.setResponseParam(JSON.toJSONString(params));
		}
		String post = HttpsUtilsTest.submitPost(apiGateway, JSON.toJSONString(apiReq));
		JSONObject result = JSON.parseObject(post);
		if (!result.get("code").equals(200) || result.get("data") == null ){
			return Result.of_error("制作失败！");
		}
		return Result.ok(result.get("data"));
	}

	/**
	 * 封装参数
	 */
	public Result packParams(DataApiUpReq dataApiUpReq, Long customerId, String apiKey) {
		//封装参数
		DataGoods dataGoods = new DataGoods();
		dataGoods.setApiOpenid(dataApiUpReq.getApiOpenId());
		dataGoods.setCreTime(DateUtils.getToday());
		dataGoods.setCrePerson(customerId+ "");
		dataGoods.setDataType(dataApiUpReq.getApiType());
		dataGoods.setUserId(customerId);
		dataGoods.setDataPicture(dataApiUpReq.getDataPicture());
		dataGoods.setAuditStatus("03");
		dataGoods.setDataStatus("01");
		dataGoods.setDelFlag("N");
		// 校验商品价格
		if (dataApiUpReq.getSecondType() != null) {
			dataGoods.setDataPrice(dataApiUpReq.getSecondType());
		}else if (dataApiUpReq.getMonthType() != null) {
			dataGoods.setDataPrice(dataApiUpReq.getMonthType());
		}else if (dataApiUpReq.getSeasonType() != null) {
			dataGoods.setDataPrice(dataApiUpReq.getSeasonType());
		}else {
			dataGoods.setDataPrice(dataApiUpReq.getYearType());
		}
		BeanUtils.copyProperties(dataApiUpReq, dataGoods);
		if (dataGoodsDao.insert(dataGoods) != 1) {
			throw ResponseException.of_error("商品添加失败！");
		}
		logger.info("############################第三方商品制作：商品添加成功！#################################");
		// 赋值api
		Long dataGoodsId = dataGoods.getDataGoodsId();
		if (dataGoodsId != null) {
			DataGoodsApi dataGoodsApi = new DataGoodsApi();

			dataGoodsApi.setDataGoodsId(dataGoodsId);
			dataGoodsApi.setApiType(dataApiUpReq.getApiType());
			dataGoodsApi.setCreTime(DateUtils.getToday());
			dataGoodsApi.setCrePerson(customerId + "");
			dataGoodsApi.setTransMode(dataApiUpReq.getApiProtocl());
			dataGoodsApi.setVersionNumber("v1.0"); // TODO 版本号
			dataGoodsApi.setDelFlag("N");
			dataGoodsApi.setApiKey(apiKey);
			BeanUtils.copyProperties(dataApiUpReq, dataGoodsApi);
			if (tDataGoodsApiDao.insert(dataGoodsApi) != 1) {
				throw ResponseException.of_error("商品api添加失败");
			}
			logger.info("############################第三方商品制作：商品api添加成功！#################################");
			// 赋值params
			Long goodsApi = dataGoodsApi.getGoodsApi();
			if (goodsApi != null) {
				// 请求参数
				List<DataGoodsApiParams> requestParam = dataApiUpReq.getRequestParam();
				if (requestParam != null || requestParam.size() != 0) {
					for (DataGoodsApiParams params : requestParam) {
						params.setGoodsApi(goodsApi);
						params.setParamsDiff("02");
						params.setDelFlag("N");
						params.setCreTime(DateUtils.getToday());
						params.setCrePerson(customerId + "");
						BeanUtils.copyProperties(dataApiUpReq, params);
						if (apiParamsDao.insert(params) != 1) {
							throw ResponseException.of_error("商品请求参数添加失败！");
						}
						logger.info("############################第三方商品制作：商品请求参数添加成功！#################################");
					}
				}
				// 响应参数
				List<DataGoodsApiParams> responseParam = dataApiUpReq.getResponseParam();
				if (responseParam != null || responseParam.size() != 0) {
					for (DataGoodsApiParams params : responseParam) {
						params.setGoodsApi(goodsApi);
						params.setParamsDiff("03");
						params.setDelFlag("N");
						params.setCreTime(DateUtils.getToday());
						params.setCrePerson(customerId + "");
						BeanUtils.copyProperties(dataApiUpReq, params);
						if (apiParamsDao.insert(params) != 1) {
							throw ResponseException.of_error("商品响应参数添加失败！");
						}
						logger.info("############################第三方商品制作：商品响应参数添加成功！#################################");
					}
				}
			}
		}
		return Result.ok();
	}

	/**
	 * 签名= joinType
	 */
	public String getSign(String joinType) {
		String s = dataBank + joinType;
		String encrypt = Md5.encrypt(s, "");
		System.out.println(encrypt);
		return encrypt;
	}

}