package com.jz.dm.mall.moduls.service.impl;


import com.jz.common.base.CurrentUser;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.UserTypeEnum;

import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;

import com.jz.common.utils.SessionUtils;
import com.jz.dm.mall.moduls.controller.customer.bean.CustomerDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.CustomerRequest;
import com.jz.dm.mall.moduls.controller.customer.bean.req.LoginRequest;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.dm.mall.moduls.entity.MallMenu;
import com.jz.dm.mall.moduls.entity.MallUserRole;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.service.MallCustomerService;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * 商城用户(TMallCustomer)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("mallCustomerService")
public class MallCustomerServiceImpl implements MallCustomerService {
    @Autowired
    private MallCustomerDao tMallCustomerDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 通过用户账号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByAccount(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByAccount(username);
        return mallCustomer;
    }

    /**
     * 通过手机号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByPhone(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByPhone(username);
        return mallCustomer;
    }

    /**
     * 注册账号
     *
     * @param
     */
    @Override
    public Result saveCustomer(CustomerRequest customerRequest) {
        // 获取验证码
        String vailCode = customerRequest.getVailCode();
        String telephone = customerRequest.getTelephone();
        String username = customerRequest.getUsername();

        Map mapUser = new HashMap();
        if (StringUtils.isNotEmpty(customerRequest.getUsername())) {
            mapUser.put("customerAccount",customerRequest.getUsername());
        }
        MallCustomerApiDto customerUser = tMallCustomerDao.selectByUser(mapUser);
        if (customerUser != null) {
            return Result.of_error("用户已注册！");
        }
        Map mapPhone = new HashMap();
        if (StringUtils.isNotEmpty(customerRequest.getTelephone())) {
            mapPhone.put("customerPhone",customerRequest.getTelephone());
        }
        MallCustomerApiDto customerPhone = tMallCustomerDao.selectByUser(mapPhone);
        if (customerPhone != null) {
            return Result.of_error("手机号已注册！");
        }

        // 从redis获取验证码
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
        Integer codeInRedis = (Integer) redisTemplate.opsForValue().get(key);
        if (codeInRedis == null) {
            return Result.of_error("验证码为空！");
        }
        String s = codeInRedis.toString();
        if (s.equals(vailCode)) {
            MallCustomer customer = new MallCustomer();
            customer.setCustomerAccount(username);
            customer.setPassword(customerRequest.getPassword());
            customer.setCustomerPhone(telephone);
            customer.setCreTime(DateUtils.getToday());
            customer.setRegisterTime(DateUtils.getToday());
            tMallCustomerDao.saveCustomer(customer);
            return Result.of_success("注册成功！");
        }
        return Result.of_error("注册失败！");
    }

    /**
     * 通过手机号查询是否已注册
     *
     * @param telephone@return
     */
    @Override
    public MallCustomer selectByPhone(String telephone) {
        MallCustomer mallCustomer = tMallCustomerDao.selectByPhoneExist(telephone);
        if (mallCustomer != null) {
            return mallCustomer;
        }
        return null;
    }

    /**
     * 修改密码
     *
     * @param customerId
     * @param password
     */
    @Override
    public void updatePassword(Long customerId, String password) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        if (mallCustomer != null) {
            MallCustomer customer = new MallCustomer();
            mallCustomer.setPassword(password);
            mallCustomer.setUptTime(DateUtils.getToday());
            mallCustomer.setUptPerson(mallCustomer.getCustomerId() + "");
            mallCustomer.setCustomerId(mallCustomer.getCustomerId());
            tMallCustomerDao.updatePassword(customer);
        }
    }

    /**
     * 通过id查找用户信息
     *
     * @param customerId
     * @return
     */
    @Override
    public MallCustomer selectByUser(Long customerId) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        return mallCustomer;
    }

    /**
     * 根据用户id，角色id查找菜单
     *
     * @param userId
     * @param roleId
     * @return
     */
    @Override
    public Result getMenu(Long userId, Long roleId) {

        //查询用户拥有的角色
        List<MallMenu> entities = tMallCustomerDao.getMenu(userId, roleId);

        // 找到所有的一级分类
        List<MallMenu> level1Menus = entities.stream().filter(sysMenuEntity ->
                sysMenuEntity.getParentId() == 0 && sysMenuEntity.getIfChild().equals("N")
        ).map((menu)->{
            menu.setChildren(getChildrens(menu,entities));
            return menu;
        }).collect(Collectors.toList());
        return Result.of_success(ResultMsg.SUCCESS, level1Menus);
    }

    /**
     * 登陆
     * @param customer
     * @return
     */
    @Override
    public Result loginByUser(LoginRequest customer) {
        // 手机登陆
        String ph = "^[1][34578]\\d{9}$";
        Map map = new HashMap();
        if (customer.getUsername().matches(ph) && StringUtils.isNotEmpty(customer.getUsername())) {
            map = new HashMap();
            map.put("customerPhone", customer.getUsername());
            MallCustomerApiDto mallCustomer = tMallCustomerDao.selectByUser(map);
            if (mallCustomer.getCustomerId() == null) {
                return Result.of_error("用户不存在，请注册！");
            }
            if (mallCustomer.getPassword().equals(customer.getPassword())) {
                // 查询当前用户有无角色
                if (StringUtils.isNotBlank(mallCustomer.getRoleId())) {
                    MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
                    BeanUtils.copyProperties(mallCustomer,mallCustomerApiDto);
                    redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
                }
                redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomer, 3, TimeUnit.DAYS);

                return Result.of_success(ResultMsg.LOGIN_SUCCESS, mallCustomer);
            }
            return Result.of_error(ResultMsg.LOGIN_ERROR);
        }
        map = new HashMap();
        map.put("customerAccount",customer.getUsername());
        MallCustomerApiDto mallCustomer = tMallCustomerDao.selectByUser(map);
        if (mallCustomer == null) {
            return Result.of_error("用户不存在，请注册！");
        }
        if (mallCustomer.getPassword().equals(customer.getPassword())) {
            // 查询当前用户有无角色
            if (StringUtils.isNotBlank(mallCustomer.getRoleId())) {
                MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
                BeanUtils.copyProperties(mallCustomer,mallCustomerApiDto);
                redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
            }
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomer, 3, TimeUnit.DAYS);
            return Result.of_success(ResultMsg.LOGIN_SUCCESS, mallCustomer);
        }
        return Result.of_error(ResultMsg.LOGIN_ERROR);
    }



    private List<MallMenu> getChildrens(MallMenu sysMenu,List<MallMenu> all) {

        List<MallMenu> children = all.stream().filter(sysMenuEntity -> {
            return sysMenuEntity.getParentId() == sysMenu.getMenuId() && sysMenuEntity.getIfChild().equals("Y");
        }).map(sysMenuEntity -> {
            //1、找到子菜单,同时递归调用
            sysMenuEntity.setChildren(getChildrens(sysMenuEntity, all));
            return sysMenuEntity;
        }).collect(Collectors.toList());

        return children;
    }


}