package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BasePageBean;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.TradeFlowEnum;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.finance.platForm.*;
import com.jz.manage.moduls.entity.FinanceCashOut;
import com.jz.manage.moduls.entity.FinanceCustomerAssets;
import com.jz.manage.moduls.entity.FinanceTradeFlow;
import com.jz.manage.moduls.mapper.*;
import com.jz.manage.moduls.service.PlatformTradeFlowInfoService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 平台交易流水(TPlatformTradeFlowInfo)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:44
 */
@Service("platformTradeFlowInfoService")
@Transactional
public class PlatformTradeFlowInfoServiceImpl implements PlatformTradeFlowInfoService {

    private static final Logger logger = LoggerFactory.getLogger(PlatformTradeFlowInfoServiceImpl.class);

    @Autowired
    private PlatformTradeFlowInfoDao platformTradeFlowInfoDao;

    @Autowired
    private FinanceCashOutDao financeCashOutDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    @Autowired
    private OrderDao orderDao;

    @Override
    public PlatFormMoneyAndDepartDto queryPlatFormMoneyAndDepart(Map map) throws Exception {
        return platformTradeFlowInfoDao.queryPlatFormMoneyAndDepart(map);
    }

    /*
     * 财务管理---我的订单
     * */
    @Override
    public PageInfoResponse<OrderCountInfoDto> findListOrderCount(OrderCountInfoReq pageBean, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<OrderCountInfoDto> pageInfoResponse = new PageInfoResponse<>();

        Map param = new HashMap();
        if (StringUtils.isNotEmpty(pageBean.getOrderTime())) {
            String orderTime = pageBean.getOrderTime();
            String startTime = orderTime.substring(0, 10);
            String endTime = orderTime.substring(11, 21);
            param.put("startTime", startTime + " 00:00:00");
            param.put("endTime", endTime + " 23:59:59");
        }
        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize());
        List<OrderCountInfoDto> list = platformTradeFlowInfoDao.findListOrderCount(param);
        PageInfo<OrderCountInfoDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /*
     * 财务管理--交易明细
     * */
    @Override
    public PageInfoResponse<TradeDetilDto> findListFinanceTradeDetail(TradeDetilRequest tradeDetilRequest, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<TradeDetilDto> pageInfoResponse = new PageInfoResponse<>();
        Map param = new HashMap();
        if (StringUtils.isNotEmpty(tradeDetilRequest.getOrderTime())) {
            String orderTime = tradeDetilRequest.getOrderTime();
            param.put("startTime", orderTime + " 00:00:00");
            param.put("endTime", orderTime + " 23:59:59");
        }

        PageHelper.startPage(tradeDetilRequest.getPageNum(), tradeDetilRequest.getPageSize());
        List<TradeDetilDto> list = platformTradeFlowInfoDao.findListFinanceTradeDetil(param);
        PageInfo<TradeDetilDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    @Override
    public  IPage<CashOutListDto>  findListFinanceCashOut(CashOutListRequest cashOutListRequest, HttpServletRequest httpRequest) throws Exception {

        IPage<CashOutListDto> page = new Page<>(cashOutListRequest.getPageNum(), cashOutListRequest.getPageSize());
        QueryWrapper<CashOutListDto> query = new QueryWrapper();

        if (StringUtils.isNotBlank(cashOutListRequest.getSearch())){
            query.and(wrapper -> wrapper.like("t3.department_name",cashOutListRequest.getSearch())
                    .or().like("t1.remark",cashOutListRequest.getSearch()));
        }
        if (StringUtils.isNotBlank(cashOutListRequest.getAuditStatus())){
            query.eq("t1.audit_status",cashOutListRequest.getAuditStatus());
        }
        query.eq("t1.del_flag","N");
        query.orderByDesc("t1.cash_out_time");
        if (StringUtils.isNotBlank(cashOutListRequest.getCasOutTime())){
            String casOutTimeStart = cashOutListRequest.getCasOutTime().substring(0, 10) + " 00:00:00";
            String casOutTimeEnd = cashOutListRequest.getCasOutTime().substring(11, 21)+ " 23:59:59";
            query.between( "t1.cash_out_time",casOutTimeStart, casOutTimeEnd);
        }
       IPage<CashOutListDto> list= platformTradeFlowInfoDao.list(page, query);
        return list;
    }

    @Override
    public CashOutInfoDto queryCashOutByid(String cashOutId) throws Exception {
        return platformTradeFlowInfoDao.queryCashOutByid(cashOutId);
    }

    /**
     * 提现管理--提现审核
     *
     * @return
     * @author Bellamy
     * @since 2020-12-06
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> uptCashOutInfoAuditByid(CashOutAuditRequest cashOutAuditRequest) throws Exception {
        FinanceCashOut cashOut = new FinanceCashOut();
        //提现id
        if (StringUtils.isNotEmpty(cashOutAuditRequest.getCashOutId())) {
            cashOut.setCashOutId(Long.valueOf(cashOutAuditRequest.getCashOutId()));
        }
        //审核意见
        if (StringUtils.isNotEmpty(cashOutAuditRequest.getRejectReason())) {
            cashOut.setRejectReason(cashOutAuditRequest.getRejectReason());
        }
        //审核状态
        cashOut.setAuditStatus(cashOutAuditRequest.getAuditStatus());
        cashOut.setAuditTime(new Date());
        //cashOut.setAuditPerson();
        //1.更新提现状态
        int len = financeCashOutDao.updateById(cashOut);
        if (len > 0) {
            logger.info("#####################修改提现状态成功#####################");
            FinanceCashOut cashOutObj = financeCashOutDao.selectById(cashOutAuditRequest.getCashOutId());
            //查询出提现金额
            String cashOutMoney = cashOutObj.getCashOutMoney().toString();
            BigDecimal txMoney = new BigDecimal(cashOutMoney); //提现金额

            if (StringUtils.isNotEmpty(cashOutObj.getAssetsId().toString())) {
                FinanceCustomerAssets assetsObj = financeCustomerAssetsDao.selectById(cashOutObj.getAssetsId().toString());
                String totalMoney = assetsObj.getTotalMoney().toString(); //总金额
                BigDecimal totalMoneyObj = new BigDecimal(totalMoney);
                String frozenMoney = assetsObj.getFrozenMoney().toString();//冻结金额
                BigDecimal frozenMoneyObJ = new BigDecimal(frozenMoney);

                //2.审核通过，修改个人资产，总金额减去提现金额，从冻结金额减去
                if (AuditStatusEnum.YSH.getValue().equals(cashOutAuditRequest.getAuditStatus())) {
                    totalMoneyObj = totalMoneyObj.subtract(txMoney);
                    frozenMoneyObJ = frozenMoneyObJ.subtract(txMoney);

                    FinanceCustomerAssets financeCustomerAssets = new FinanceCustomerAssets();
                    financeCustomerAssets.setAssetsId(cashOutObj.getAssetsId());
                    financeCustomerAssets.setTotalMoney(totalMoneyObj);
                    financeCustomerAssets.setFrozenMoney(frozenMoneyObJ);
                    //financeCustomerAssets.setUptPerson();
                    financeCustomerAssets.setUptTime(new Date());
                    financeCustomerAssetsDao.updateById(financeCustomerAssets);
                    logger.info("#####################提现通过---修改资产金额态成功#####################");

                    //3.商城流水添加一条流水记录
                    FinanceTradeFlow financeTradeFlow = new FinanceTradeFlow();
                    financeTradeFlow.setCreTime(new Date());
                    financeTradeFlow.setTradeMoney(txMoney);
                    financeTradeFlow.setAssetsId(cashOutObj.getAssetsId());
                    financeTradeFlow.setTradeType(TradeFlowEnum.TX.getValue());
                    financeTradeFlow.setTradeFlowNumber("123132");
                    financeTradeFlow.setBalanceMoney(assetsObj.getUseMoney());
                    //financeTradeFlow.setCashOutId();
                    //financeTradeFlow.setCrePerson();
                    financeTradeFlowDao.insert(financeTradeFlow);
                    logger.info("#####################商城提现流水记录添加成功#####################");
                } else if (AuditStatusEnum.WTG.getValue().equals(cashOutAuditRequest.getAuditStatus())) {
                    //3.审核不通过，修改个人资产，可用金额加上提现金额，从冻结金额减去
                    String useMoney = assetsObj.getUseMoney().toString(); //可用金额
                    BigDecimal useMoneyObj = new BigDecimal(useMoney);
                    useMoneyObj = useMoneyObj.add(txMoney);

                    frozenMoneyObJ = frozenMoneyObJ.subtract(txMoney);

                    FinanceCustomerAssets financeCustomerAssets = new FinanceCustomerAssets();
                    financeCustomerAssets.setAssetsId(cashOutObj.getAssetsId());
                    financeCustomerAssets.setUseMoney(useMoneyObj);
                    financeCustomerAssets.setFrozenMoney(frozenMoneyObJ);
                    //financeCustomerAssets.setUptPerson();
                    financeCustomerAssets.setUptTime(new Date());
                    financeCustomerAssetsDao.updateById(financeCustomerAssets);
                    logger.info("#####################提现审核拒绝---修改资产金额态成功#####################");
                }
            }
            return Result.ok();
        } else {
            return Result.error("提交失败");
        }
    }

    @Override
    public Result<MoneyAndOrderTotalDto> getMoneyAndOrderTotalDto(String time) {
        Map map = new HashMap();
        if (StringUtils.isNotBlank(time)){
            String startTime = time.substring(0, 10);
            String endTime = time.substring(11, 21);
            map.put("startTime", startTime + " 00:00:00");
            map.put("endTime", endTime + " 23:59:59");
        }
        MoneyAndOrderTotalDto dto = orderDao.getMoneyAndOrderTotalDto(map);
        if (StringUtils.isNotBlank(dto.getOrderTotal())) {
            return Result.of_error("查询失败！");
        }
        String orderTotal = dto.getOrderTotal();
        BigDecimal transactionMoney = dto.getTransactionMoney();
        BigDecimal divide = transactionMoney.divide(new BigDecimal(orderTotal), 2, BigDecimal.ROUND_DOWN);
        dto.setMeanTransactionMoney(divide.toString());
        return Result.of_success(dto);
    }
}